module Fedi.Types.Helpers where

import Data.Text qualified as T
import Fedi.Types
import Fedi.UserDetails

-- | An empty activitypub Object.
emptyObject :: Object ()
emptyObject =
  Object
    { id = Nothing
    , otype = ()
    , content = Nothing
    , published = Nothing
    , replies = Nothing
    , attachment = Nothing
    , attributedTo = Nothing
    , tag = Nothing
    , to = Nothing
    , cc = Nothing
    , inReplyTo = Nothing
    , url = Nothing
    , name = Nothing
    , icon = Nothing
    , image = Nothing
    , preview = Nothing
    , summary = Nothing
    , updated = Nothing
    , mediaType = Nothing
    }

-- | Create an activitypub Actor.
makeActor :: UserDetails -> Actor
makeActor details =
  let
    actor = actorUrl details
  in
    ActorPerson $
      emptyObject
        { id = Just $ ObjectId actor
        , otype =
            TypePerson
              { preferredUsername = details.username
              , inbox = Link $ actor <> "/inbox"
              , outbox = Link $ actor <> "/outbox"
              , following = Link $ actor <> "/following"
              , followers = Link $ actor <> "/followers"
              , publicKey =
                  PublicKey
                    { pkid = actor <> "#main-key"
                    , owner = actor
                    , publicKeyPem = details.publicPem
                    }
              }
        , url = Nothing -- details.url
        , name = Just $ StringName details.name
        , icon = Just $ makeImage details.icon
        , image = Just $ makeImage details.image
        , summary = Just $ T.pack details.summary
        }

makeCreateNote :: Note -> Create
makeCreateNote note =
  emptyObject
    { id = (\oid -> ObjectId $ oid.unwrap <> "/create") <$> note.id
    , otype =
        TypeActivity
          { actor = maybe (Link "") getAttributedTo note.attributedTo
          , atype = TypeCreate note
          , target = Nothing
          , origin = Nothing
          }
    }

-- | Create an user's empty 'Note'.
emptyUserNote :: Url -> Note
emptyUserNote actor =
  emptyObject
    { otype = emptyTypeNote
    , attributedTo = Just (LLink $ Link actor)
    , to = Just [Link "https://www.w3.org/ns/activitystreams#Public"]
    , cc = Just [Link $ actor <> "/followers"]
    }

-- | An empty 'Note'.
emptyTypeNote :: TypeNote
emptyTypeNote =
  TypeNote
    { likes = emptyUnorderedCollection
    , shares = emptyUnorderedCollection
    , replies = emptyUnorderedCollection
    , sensitive = False
    }

-- | Create an activitypub Image.
makeImage :: Url -> Image
makeImage link =
  emptyObject
    { otype = TypeImage
    , mediaType = Just ("image/png" :: MediaType)
    , url = Just link
    }

-- | An empty 'Collection'.
emptyUnorderedCollection :: Collection a
emptyUnorderedCollection =
  emptyObject
    { otype =
        CollectionType
          { ctype =
              UnorderedCollectionType
                { items = []
                }
          , first = Nothing
          , last = Nothing
          , current = Nothing
          }
    }

-- | An empty 'OrderedCollection'.
emptyOrderedCollection :: OrderedCollection a
emptyOrderedCollection =
  emptyObject
    { otype =
        CollectionType
          { ctype =
              OrderedCollectionType
                { orderedItems = []
                }
          , first = Nothing
          , last = Nothing
          , current = Nothing
          }
    }

-- | Create an empty 'OrderedCollectionPage'.
emptyOrderedCollectionPage :: Url -> OrderedCollectionPage a
emptyOrderedCollectionPage url =
  emptyObject
    { otype =
        CollectionType
          { ctype =
              OrderedCollectionPageType
                { partOf = url
                , prev = Nothing
                , next = Nothing
                , porderedItems = []
                }
          , first = Nothing
          , last = Nothing
          , current = Nothing
          }
    }

makeAccept :: Follow -> Url -> Accept
makeAccept theirFollow myfollowId =
  emptyObject
    { id = Just $ ObjectId myfollowId
    , otype =
        TypeActivity
          { actor = theirFollow.otype.actor
          , atype = TypeAccept
            { object = ActivityFollow theirFollow
            }
          , target = Nothing
          , origin = Nothing
          }
    }
